from glob import glob
import os
import pandas as pd

def lnc_filter(data,col):
    data = data.dropna().drop_duplicates()
    lncrans = pd.read_csv("data/LncSEA+NONCODE+GENCODE.lnc.txt")
    lncrans = set(lncrans["LncRNA"])
    print("---------------Shape: ---------------",data.shape)
    data = data[data[col].apply(lambda x:x in lncrans)]
    print("---------------Shape: ---------------",data.shape)
    return data

def expand_row(data,col,split=","):
    r = data[col].str.split(split,expand=True)
    r = r.stack().reset_index(level=1, drop=True).dropna().rename(col)
    data = data.drop([col],axis=1).join(r)
    return data

def run(data_path, detail_path):
    ## Enhancer ##
    # Enhancer)
    print("------------Enhancer------------")
    data = pd.read_csv(data_path[0],sep="\t")
    columns = ["cell_id","se_chr","se_start","se_end","target_gene","method"]
    te_gene_overlap = lnc_filter(expand_row(data[["cell_id","te_chr","te_start","te_end","te_gene_overlap"]],"te_gene_overlap"),"te_gene_overlap")
    te_gene_overlap["method"] = "Overlap"
    te_gene_overlap.columns = columns
    te_gene_proximal = lnc_filter(expand_row(data[["cell_id","te_chr","te_start","te_end","te_gene_proximal"]],"te_gene_proximal"),"te_gene_proximal")
    te_gene_proximal["method"] = "Proximal"
    te_gene_proximal.columns = columns
    te_gene_closest = lnc_filter(expand_row(data[["cell_id","te_chr","te_start","te_end","te_gene_closest"]],"te_gene_closest"),"te_gene_closest")
    te_gene_closest["method"] = "Closest"
    te_gene_closest.columns = columns
    te_target_gene = pd.concat([te_gene_overlap,te_gene_proximal,te_gene_closest])

    info = pd.read_csv(data_path[1],sep="\t")
    data = te_target_gene.merge(info,on="cell_id")
    data = data.query("Species != 'Mouse'")
    data["Set"] = data["cell_id"] + "_" + data["cell_type"] + "_" + data["cell_name"]
    data["Version"] = "hg38"
    data = data[["Set","target_gene","se_chr","se_start","se_end","Version","method"]]
    data.columns = ["Set","LncRNA","D1_chr","D2_start","D3_end","Version","Sub_Class"]
    data["Class"] = "Enhancer"

    e_data = [data]
    for sub_class in data["Sub_Class"].unique():
        detail = pd.read_csv(f"data_output/detail_Super_Enhancer_{sub_class}.txt")
        detail["Class"] = "Enhancer"
        e_data.append(detail)

    data = pd.concat(e_data)
    data.to_csv(detail_path[0], index=False)

run(snakemake.input, snakemake.output)
