import pandas as pd


def lnc_filter(data):
    data = data.dropna().drop_duplicates()
    lncrans = pd.read_csv("data/LncSEA+NONCODE+GENCODE.lnc.txt")
    lncrans = set(lncrans["LncRNA"])
    print("---------------Shape: ---------------", data.shape)
    data = data[data.LncRNA.apply(lambda x: x in lncrans)]
    print("---------------Shape: ---------------", data.shape)
    return data


def expand_row(data, col, split=","):
    r = data[col].str.split(split, expand=True)
    r = r.stack().reset_index(level=1, drop=True).dropna().rename(col)
    data = data.drop([col], axis=1).join(r)
    return data


def run(data_path, detail_path):
    ## ENCORI ##
    # ENCORI
    print("------------ENCORI------------")
    data = pd.read_csv(data_path[0], header=3, sep="\t")
    data = data[
        [
            "pairGeneName",
            "geneName",
            "pairGeneType",
            "geneType",
            "interactionLocus",
            "CellLine/Tissue",
        ]
    ]
    data.columns = [
        "Set",
        "LncRNA",
        "D1_GeneTypeA",
        "D2_GeneTypeB",
        "D3_interactionLocus",
        "D4_CellLine_Tissue",
    ]
    types = [
        "antisense",
        "sense_overlapping",
        "sense_intronic",
        "lincRNA",
        "processed_transcript",
        "bidirectional_promoter_lncRNA",
        "3prime_overlapping_ncRNA",
        "non_coding",
        "macro_lncRNA",
    ]
    data = data[data["D2_GeneTypeB"].str.contains("|".join(types))]
    data["Sub_Class"] = "ENCORI"
    data["Class"] = "RNA_RNA_Interaction"
    data.to_csv(detail_path[0], index=False)


run(snakemake.input, snakemake.output)
